package com.caimei365.wechat.service.impl;

import com.caimei365.wechat.service.WeChatTokenService;
import com.caimei365.wechat.utils.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

@Slf4j
@Service
public class WeChatTokenServiceImpl implements WeChatTokenService {
    /**
     * 1. 获取应用的accessToken，
     * 获取接口：https://qyapi.weixin.qq.com/cgi-bin/service/get_suite_token
     * 官方文档：https://developer.work.weixin.qq.com/document/path/90600
     * 请求方式：POST
     * 参数：{
     * "suite_id":"wwddddccc7775555aaa" ,
     * "suite_secret": "ldAE_H9anCRN21GKXVfdAAAAAAAAAAAAAAAAAA",
     * "suite_ticket": "Cfp0_givEagXcYJIztF6sfbdmIZCmpaR8ZBsvJEFFNBrWmnD5-CGYJ3_NhYexMyw"
     * }
     * 返回结果：{
     * "errcode":0,
     * "errmsg":"ok",
     * "suite_access_token":"61W3mEpU66027wgNZ_MhGHNQDHnFATkDa9-2llMBjUwxRSNPbVsMmyD-yq8wZETSoE5NQgecigDrSHkPtIYA",
     * "expires_in":7200
     * }
     * <p>
     * 2.获取预授权码 pre_auth_code
     * 获取接口：https://qyapi.weixin.qq.com/cgi-bin/service/get_pre_auth_code?suite_access_token=SUITE_ACCESS_TOKEN
     * 官方文档：https://developer.work.weixin.qq.com/document/path/90601
     * 请求方式：GET
     * 返回结果：{w'q
     * "errcode":0 ,
     * "errmsg":"ok" ,
     * "pre_auth_code":"Cx_Dk6qiBE0Dmx4EmlT3oRfArPvwSQ-oa3NL_fwHM7VI08r52wazoZX2Rhpz1dEw",
     * "expires_in":1200
     * }
     * <p>
     * 3.获取企业永久授权码
     * 获取接口：https://qyapi.weixin.qq.com/cgi-bin/service/get_permanent_code?suite_access_token=SUITE_ACCESS_TOKEN
     * 官方文档：https://developer.work.weixin.qq.com/document/path/90603
     * 请求方式：POST
     * 返回结果：{
     * 	"errcode":0,
     * 	"errmsg":"ok",
     * 	"access_token": "xxxxxx",
     * 	"expires_in": 7200,
     * 	"permanent_code": "xxxx",
     * 	"dealer_corp_info":{
     * 		"corpid": "xxxx",
     * 		"corp_name": "name"
     *    },
     * 	"auth_corp_info":{
     * 		"corpid": "xxxx",
     * 		"corp_name": "name",
     * 		"corp_type": "verified",
     * 		"corp_square_logo_url": "yyyyy",
     * 		"corp_user_max": 50,
     * 		"corp_full_name":"full_name",
     * 		"verified_end_time":1431775834,
     * 		"subject_type": 1,
     * 		"corp_wxqrcode": "zzzzz",
     * 		"corp_scale": "1-50人",
     * 		"corp_industry": "IT服务",
     * 		"corp_sub_industry": "计算机软件/硬件/信息服务"
     *    },
     * 	"auth_info":
     *    {
     * 		"agent" :
     * 		[
     *            {
     * 				"agentid":1,
     * 				"name":"NAME",
     * 				"round_logo_url":"xxxxxx",
     * 				"square_logo_url":"yyyyyy",
     * 				"appid":1,
     * 				"auth_mode":1,
     * 				"is_customized_app":false,
     * 				"auth_from_thirdapp":false,
     * 				"privilege":
     *                {
     * 					"level":1,
     * 					"allow_party":[1,2,3],
     * 					"allow_user":["zhansan","lisi"],
     * 					"allow_tag":[1,2,3],
     * 					"extra_party":[4,5,6],
     * 					"extra_user":["wangwu"],
     * 					"extra_tag":[4,5,6]
     *                },
     * 				"shared_from":
     *                {
     * 					"corpid":"wwyyyyy",
     * 					"share_type": 1
     *                }
     *            },
     *            {
     * 				"agentid":2,
     * 				"name":"NAME2",
     * 				"round_logo_url":"xxxxxx",
     * 				"square_logo_url":"yyyyyy",
     * 				"appid":5,
     * 				"shared_from":
     *                {
     * 					"corpid":"wwyyyyy",
     * 					"share_type": 0
     *                }
     *            }
     * 		]
     *    },
     * 	"auth_user_info":
     *    {
     * 		"userid":"aa",
     * 		"open_userid":"xxxxxx",
     * 		"name":"xxx",
     * 		"avatar":"http://xxx"
     *    },
     * 	"register_code_info":
     *    {
     * 		"register_code":"1111",
     * 		"template_id":"tpl111",
     * 		"state":"state001"
     *    },
     * 	"state":"state001"
     * }
     *
     */

    @Override
    public String getAccessToken(String suiteTicket) {
        String response = "";
        try {
            Map<String, String> postMap = new HashMap<>();
            postMap.put("suite_id", "dkf01ed803e47f9a48");
            postMap.put("suite_secret", "tiEVBsRvpcm-8RC5AixCyY6N3BcCDrrVBwpfMMvvd8Q");
            postMap.put("suite_ticket", suiteTicket);
            response = HttpUtil.sendPost("https://qyapi.weixin.qq.com/cgi-bin/service/get_suite_token", postMap);
        } catch (Exception e) {
            log.error("获取accessToken失败------------------------------------->" + e);
            throw new RuntimeException(e);
        }
        return response;
    }

    @Override
    public String getPreAuthCode(String suiteAccessToken) {
        String preAuthCode = "";
        try {
            preAuthCode = HttpUtil.sendGet("https://qyapi.weixin.qq.com/cgi-bin/service/get_pre_auth_code?suite_access_token=" + suiteAccessToken);
        } catch (Exception e) {
            log.error("获取预授权码失败------------------------------------->" + e);
            throw new RuntimeException(e);
        }
        return preAuthCode;
    }

    @Override
    public String getPermanentCode(String suiteAccessToken, String preAuthCode) {
        String permanentCode = "";
        try {
            Map<String, String> postMap = new HashMap<>();
            postMap.put("auth_code", preAuthCode);
            permanentCode = HttpUtil.sendPost(" https://qyapi.weixin.qq.com/cgi-bin/service/get_permanent_code?suite_access_token=" + suiteAccessToken, postMap);
        } catch (Exception e) {
            log.error("获取企业永久授权码------------------------------------->" + e);
            throw new RuntimeException(e);
        }
        return permanentCode;
    }

    public static void unzip(String zipFilePath, String destDirectory) throws IOException {
        File destDir = new File(destDirectory);
        if (!destDir.exists()) {
            destDir.mkdir();
        }
        ZipInputStream zipIn = new ZipInputStream(new FileInputStream(zipFilePath));
        ZipEntry entry = zipIn.getNextEntry();
        while (entry != null) {
            String filePath = destDirectory + File.separator + entry.getName();
            if (!entry.isDirectory()) {
                extractFile(zipIn, filePath);
            } else {
                File dir = new File(filePath);
                dir.mkdir();
            }
            zipIn.closeEntry();
            entry = zipIn.getNextEntry();
        }
        zipIn.close();
    }

    private static void extractFile(ZipInputStream zipIn, String filePath) throws IOException {
        File file = new File(filePath);
        file.getParentFile().mkdirs();
        try (FileOutputStream fos = new FileOutputStream(file)) {
            byte[] bytes = new byte[4096];
            int length;
            while ((length = zipIn.read(bytes)) != -1) {
                fos.write(bytes, 0, length);
            }
        }
    }

    public static void main(String[] args) {
        String zipFilePath = "C:\\Users\\Administrator\\Desktop\\wcc.zip";
        String destDirectory = "C:\\Users\\Administrator\\Desktop";
        try {
            unzip(zipFilePath, destDirectory);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
